/*
    Copyright (C) 2004 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <cryptfuncs.hpp>

std::string sha1_hash( const char *pass )
{
	std::string hash;

	unsigned char md_value[EVP_MAX_MD_SIZE] = { '\0' };
	char temp[2] = { '\0' };
	EVP_MD_CTX mdctx;
	unsigned int md_len;

	// initialize the CTX
	EVP_MD_CTX_init( &mdctx );

	// initialize the SHA1 algorithm
	EVP_DigestInit_ex( &mdctx, EVP_sha1(), NULL );

	// hash the pass
	EVP_DigestUpdate( &mdctx, pass, strlen( pass ) );

	// finalize the hash
	EVP_DigestFinal_ex( &mdctx, md_value, &md_len );

	// clean the ctx up
	EVP_MD_CTX_cleanup( &mdctx );

	for( int i = 0; i < md_len; i++ )
	{
		bzero( temp, 2 );
		sprintf( temp, "%02x", md_value[i] );
		hash +=  temp;
	}

	return hash;
}

bool bf_crypt_file( const char *inFile, const char *outFile, const char *pass, char encordec, std::string &error )
{
	unsigned char in[1042];
	unsigned char out[1024 + EVP_MAX_BLOCK_LENGTH];

	// initialization vector
	unsigned char ivec[8] = { 7, 0, 2, 1, 4, 0, 66, 0 };

	// this is the encryption signature.  If you try to decrypt a file and it doesnt' begin
	// with this it will fail
	unsigned char encsig[25] = { 0, 2, 144, 4, 3, 4, 5, 6, 3, 12, 11, 34, 5, 6, 7, 2, 0, 3, 4, 3, 4, 3, 2, 4, 5 };
	unsigned char checksig[25] = { '\0' };

	int size = 0;
	int outlen = 0;

	EVP_CIPHER_CTX ctx;
	EVP_CIPHER_CTX_init( &ctx );

	// open the input and the output files
	FILE *ifile = fopen( inFile, "rb" );
	FILE *ofile = fopen( outFile, "wb" );

	if( !ifile ) error = "Could not open journal file for reading";
	if( !ofile ) error = "Could not open temp file for writing";

	if( !ifile || !ofile ) return false;

	int type;

	if( encordec == 'd' )
	{
		type = 0;

		// check for the encrypted signature
		fread( checksig, 1, 25, ifile );
		if( strcmp( (char *)checksig, (char *)encsig ) != 0 )
		{
			error = "Journal file is either corrupt or the password is incorrect.";
			return false;
		}
	}
	else {
		type = 1;

		// write the encrypted signature
		fwrite( encsig, 1, 25, ofile );
	}

	// initialize the encrypt functions to use blowfish
	if( !EVP_CipherInit_ex( &ctx, EVP_bf_cbc(), NULL, NULL, NULL, type ) )
	{
		error = "OpenSSL error setting ctx key.";
		return false;
	}

	// set the blowfish key length
	EVP_CIPHER_CTX_set_key_length( &ctx, strlen( pass ) );

	// set the key and the initialization vector
	if( !EVP_CipherInit_ex( &ctx, NULL, NULL, (unsigned char *)pass, ivec, type ) )
	{
		error = "OpenSSL error setting ctx key.";
		return false;
	}


	while( !feof( ifile ) )
	{
		// read in data from the in file
		size = fread( in, 1, 1024, ifile );

		if( size )
		{
			// encrypt/decrypt the data
			if( !EVP_CipherUpdate( &ctx, out, &outlen, in, size ) )
			{
				error = "OpenSSL error while decrypting/encrypting journal";
				return false;
			}

			// write the new data to the output file
			fwrite( out, 1, outlen, ofile );
		}
	}

	// finalize the encryption (this adds padding where it is needed
	if( !EVP_CipherFinal_ex( &ctx, out, &outlen ) )
	{
		error = "OpenSSL error while decrypting/encrypting journal";
		return false;
	}

	// write the finalized/padded info to the file
	fwrite( out, 1, outlen, ofile );

	EVP_CIPHER_CTX_cleanup( &ctx );

	// close the input and the output files
	fclose( ifile );
	fclose( ofile );
	return true;
}

void shred_file( const char *file )
{
	std::fstream in( file, std::ios::binary );
	if( !in.is_open() ) return;

	in.seekg( 0, std::ios::end );
	unsigned int fileLength = in.tellg();
	in.seekg( 0, std::ios::beg );
	in.close();

	for( int i = 0; i < fileLength; i++ )
	{
		in.put( '\0' );
	}

	in.close();

	unlink( file );
}
