/*
    Copyright (C) 2004 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "bfcryptfile.hpp"
#include "cryptfuncs.hpp"
#include <iostream>
#include <string>
#include <stdlib.h>
#include <sstream>

using namespace std;
using namespace aolib;

void error( std::string error )
{
	cout << error << endl;
	exit( -1 );
}

int main( int argc, char **argv )
{
	string username, password, fileLocation;
	string homedir = getenv( "HOME" );
	homedir += "/.gtkjournal";
	string err;
	stringstream tmp;
	int r;

	srand( time( 0 ) );
	tmp << "/tmp/.tmp__gtkjf__" << rand() << rand();

	char checkPass[1024];
	char checkUser[1024];

	ifstream in( ( homedir + "/.passwd" ).c_str(), ios::in );

	cout << "This program will convert a GtkJournal 0.3.2a journal file to\n";
	cout << "a 0.3.3b GtkJournal File. It will _NOT_ remove the old journal\n";
	cout << "files, which are located in ~/.gtkjournal.  Once you feel that\n";
	cout << "the converter has converted the files correctly, please \n";
	cout << "rm -rf ~/.gtkjournal.\n\n";
	cout << "Enter the current journal file's username: ";

	cin >> username;

	cout << "Enter the current journal file's password: ";

	cin >> password;

	if( !in.is_open() ) error( "Could not open password file for reading." );
	in.getline( checkUser, 1024 );
	in.getline( checkPass, 1024 );

	if( strcmp( checkUser, username.c_str() ) != 0 ) error( "The username you specified is incorrect." );
	if( strcmp( checkPass, sha1_hash( password.c_str() ).c_str() ) != 0 ) error( "The password you specified is incorrect." );
	in.close();

	cout << "Enter a location for the newly converted file (ie: journal.gtkj): ";
	cin >> fileLocation;

	if( !bf_crypt_file( ( homedir + "/journal.enc" ).c_str(), tmp.str().c_str(), password.c_str(), 'd', err ) ) error( err );

	try {
		aolib::BFCryptFile::encryptFile( username, password, tmp.str(), fileLocation );
	}
	catch( aolib::BFCryptException ex )
	{
		error( ex.what() );
	}

	cout << "Conversion successfull, journal file saved to \"" + fileLocation + "\".\n";
	aolib::BFCryptFile::shredFile( tmp.str().c_str() );
	cout << "Done.\n\n";

	return 0;
}
