/*
    Copyright (C) 2004 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <gtkmm.h>
#include "informationdialog.hpp"

namespace aolib
{
	InformationDialog::InformationDialog()
	: Gtk::Dialog( "Personal Information" ), informationLabel( "Personal Information" ), firstNameLabel( "First Name:   ", Gtk::ALIGN_LEFT ),
	lastNameLabel( "Last Name:   ", Gtk::ALIGN_LEFT ), usernameLabel( "Username:   ", Gtk::ALIGN_LEFT ),
	passwordLabel( "Password:   ", Gtk::ALIGN_LEFT ), verifyLabel( "Verify Password:   ", Gtk::ALIGN_LEFT ),
	okButton( Gtk::Stock::OK ), mainBox( true, 0 )
	{
		// lay out the components so they look nice
		_layoutComponents();
		add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );

		get_action_area()->pack_start( okButton );
		okButton.signal_clicked().connect( SigC::slot( *this, &InformationDialog::_okHandler ) );
		verifyEntry.signal_activate().connect( SigC::slot( *this, &InformationDialog::_okHandler ) );
		signal_response().connect( SigC::slot( *this, &InformationDialog::_responseHandler ) );

		// center the window
		set_position( Gtk::WIN_POS_CENTER );
		set_has_separator();

		// show the windows children
		show_all_children();
		set_resizable( false );
	}

	bool InformationDialog::_checkInformation()
	{
		if( firstNameEntry.get_text() == "" || lastNameEntry.get_text() == "" )
		{
			Gtk::MessageDialog dialog( *this, "Please enter both your first and last names.", Gtk::MESSAGE_WARNING );
			dialog.run();
			return false;
		}

		if( usernameEntry.get_text() == "" || passwordEntry.get_text() == "" )
		{
			Gtk::MessageDialog dialog( *this, "Please a username and password.", Gtk::MESSAGE_WARNING );
			dialog.run();
			return false;
		}

		if( passwordEntry.get_text() != verifyEntry.get_text() )
		{
			Gtk::MessageDialog dialog( *this, "Password and verification do not match.", Gtk::MESSAGE_WARNING );
			dialog.run();
			return false;
		}

		return true;
	}

	void InformationDialog::_okHandler()
	{
		if( _checkInformation() )
		{
			response( Gtk::RESPONSE_OK );
		}
	}

	void InformationDialog::_responseHandler( int response )
	{
		hide();
	}

	void InformationDialog::_layoutComponents()
	{
		// set up all the components so they look nice
		get_vbox()->pack_start( mainBox );

		mainBox.set_border_width( 5 );

		// this will make it so you can't see what's being typed in the
		// password box
		passwordEntry.set_visibility( false );
		verifyEntry.set_visibility( false );

		firstNameEntry.set_width_chars( 33 );
		lastNameEntry.set_width_chars( 33 );
		passwordEntry.set_width_chars( 33 );
		usernameEntry.set_width_chars( 33 );
		verifyEntry.set_width_chars( 33 );

		passwordEntry.set_max_length( 33 );
		verifyEntry.set_max_length( 33 );

		mainBox.pack_start( informationLabel );

		firstNameBox.pack_start( firstNameLabel );
		firstNameBox.pack_start( firstNameEntry, false, true );

		mainBox.pack_start( firstNameBox );

		lastNameBox.pack_start( lastNameLabel );
		lastNameBox.pack_start( lastNameEntry, false, true );

		mainBox.pack_start( lastNameBox );

		usernameBox.pack_start( usernameLabel );
		usernameBox.pack_start( usernameEntry, false, true );

		mainBox.pack_start( usernameBox );

		passwordBox.pack_start( passwordLabel );
		passwordBox.pack_start( passwordEntry, false, true );

		mainBox.pack_start( passwordBox );

		verifyBox.pack_start( verifyLabel );
		verifyBox.pack_start( verifyEntry, false, true );

		mainBox.pack_start( verifyBox );
	}
}
