/*
    Copyright (C) 2004 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef JOURNAL_WINDOW_HPP
#define JOURNAL_WINDOW_HPP

#include <gtkmm.h>
#include "config.h"
#ifdef HAVE_GTKSPELL
extern "C" {
	#include <gtkspell/gtkspell.h>
	#include <gtk/gtk.h>
}
#endif

#include <iostream>
#include <map>
#include <iomanip>
#include <sstream>
#include <fstream>
#include <string>

#include <sys/types.h>
#include <dirent.h>

#include "icon.h"
#include "bfcryptfile.hpp"
#include "logindialog.hpp"
#include "datefuncs.hpp"
#include "informationdialog.hpp"
#include "globals.hpp"
#include "sqlitemanager.hpp"

namespace gjournal
{
	class JournalFile;

	/**
	 * This is the main Window of the application.  It takes care of loading, saving, and displaying the entries.
	 * Users can also update their settings from within this widget.
	 * @author Adam Olsen
	**/
	class JournalWindow: public Gtk::Window
	{
		public:
			JournalWindow( const char *loadFile );
			virtual ~JournalWindow();

		private:
			friend class JournalFile;

			Gtk::Label dateLabel;
			Gtk::TextView journalEntry;
			Gtk::Calendar calendar;
			Gtk::VBox mainBox, calendarBox;
			Gtk::HBox centerBox;
			Gtk::MenuBar menuBar;
			Gtk::Menu fileMenu, editMenu, helpMenu;
			Gtk::ScrolledWindow window;
			Gtk::Toolbar toolBar;
			Gtk::Statusbar statusBar;
			SigC::Connection modified;
			std::string journalFile, tempFile;
			JournalFile *jf;

			bool savedFlag, loading;

			// Lays the separate widgets out so they look nice
			virtual void _layoutComponents();

			// Jumps to today's journal entry
			virtual void _todayHandler();

			// Displays an about box
			virtual void _aboutHandler();

			// Encrypts the journal file on disk
			virtual void _saveHandler();

			// just calls _exitHandler()
			virtual bool _closeHandler( GdkEventAny *event );

			// displays a Message and free()s the SQL error returned by sqlite
			virtual void _sqlError( std::string error, bool exit = false );

			// Checks to see if the Journal has been saved.  If not, it asks you if you are
			// sure you want to quit without saving
			virtual void _exitHandler();

			// Connects to the SQLite database
			virtual void _dbConnect();

			// loads the journal entry specified
			virtual void _loadEntry( int month, int mday, int year );

			// saves the entry in the buffer
			virtual void _saveCurrentEntry();

			// loads the entry when you double click on a day in the calendar
			virtual void _daySelectHandler();

			// the following are to recieve any change events in the TextBuffer so
			// that we know if the journal has been changed or not
			virtual void _journalEntryChangeHandler();

			virtual void _newFileHandler();
			virtual void _shredTempFile( std::string file );
			virtual void _openHandler();
			virtual void _saveAsHandler();
			virtual void _preferencesHandler();
			virtual void _checkForTemps();
			virtual void _checkForOldFileFormat();
			virtual void _markMonthDays();

			// contains the date string of the current entry
			std::string currentEntry;

			// the sqlite database handle
			sqlitem::SQliteManager sql;
	};
}

#include "journalfile.hpp"

#endif // JOURNAL_WINDOW_HPP
