/*
    Copyright (C) 2004 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef SQLITE_MANAGER_HPP
#define SQLITE_MANAGER_HPP

#include <sqlite.h>
#include <iostream>
#include <string>
#include <map>
#include <list>

namespace sqlitem
{
	class SQliteResult;

	/**
	 * Provides simple C++ bindings to the C SQlite interface.
	 *
	 * @author Adam Olsen
	 **/
	class SQliteManager
	{
		public:
			SQliteManager() { connected = false; }
			bool connect( std::string db );
			bool isConnected() { return connected; }
			void disconnect();
			SQliteResult query( std::string query );
			int go( std::string query );
			static std::string addSlashes( std::string text );

		private:
			sqlite *dbh;
			bool connected;

		friend class SQliteResult;
	};

	/**
	 * SQlite Exceptions base class
	**/
	class SQliteException
	{
		protected:
			std::string message;
			SQliteException() {};
		public:
			SQliteException( std::string message ) { this->message = message; }
			std::string what() { return message; }
	};

	/**
	 * Connection exception.  Will be thrown when a connection to an SQlite database fails
	**/
	class SQliteConnectionException: public SQliteException
	{
		public:
			SQliteConnectionException( std::string message ) { this->message = message; }
			SQliteConnectionException( const char *message ) { this->message = message; }
	};

	/**
	 * Query Exception.  Will be thrown when there's an error in a query - IE, syntax error
	 * or table doesn't exist.. etc.
	**/
	class SQliteQueryException: public SQliteException
	{
		public:
			SQliteQueryException( std::string message ) { this->message = message; }
			SQliteQueryException( const char *message ) { this->message = message; }
	};

}

#include "sqliteresult.hpp"

#endif // SQLITE_MANAGER_HPP
